<?php

declare(strict_types=1);

namespace hood\vk\loggers;

use DateTime;
use Monolog\logger;
use Monolog\Handler\StreamHandler;
use Jasny\ErrorHandler;
use hood\vk\core,
    hood\vk\traits\singleton;

/**
 * Журналист Jasmo
 * 
 * Основан на "monolog/monolog" и "jasny/error-handler" 
 * Jasmo = Jasny + Monolog
 * 
 * @see Monolog\logger
 * @see Jasny\ErrorHandler
 * 
 * @package hood\vk\loggers
 * @author Arsen Mirzaev Tatyano-Muradovich <red@hood.su>
 */
final class jasmo extends logger
{
    use singleton;

    /**
     * Экземпляр класса журналиста
     *
     * @var logger
     */
    public static $logger;

    // /**
    //  * Экземпляр класса обработчика ошибок
    //  *
    //  * @var ErrorHandler 
    //  */
    // public ErrorHandler $handler;

    public static function post($file = null): ?jasmo
    {
        $file = $file ?? date_format(new DateTime(core::init()->timezone), 'Y.m.d');

        /** 
         * Создание логгера по спецификации PSR-3 (Monolog)
         * 
         * @param string Название канала логирования
         */
        self::$logger = new logger(__CLASS__);

        /** 
         * Создание обработчиков (порядок обязателен) 
         */
        self::$logger->pushHandler(new StreamHandler(core::init()->path['logs'] . "/${file}-INFO.log", logger::INFO, false));               // Инфомация о процессе работы
        self::$logger->pushHandler(new StreamHandler(core::init()->path['logs'] . "/${file}-NOTICE.log", logger::NOTICE, false));           // Уведомления
        self::$logger->pushHandler(new StreamHandler(core::init()->path['logs'] . "/${file}-WARNING.log", logger::WARNING, false));         // Предупреждения
        self::$logger->pushHandler(new StreamHandler(core::init()->path['logs'] . "/${file}-ERROR.log", logger::ERROR, false));             // Ошибки
        self::$logger->pushHandler(new StreamHandler(core::init()->path['logs'] . "/${file}-CRITICAL.log", logger::CRITICAL, false));       // Критические ошибки
        self::$logger->pushHandler(new StreamHandler(core::init()->path['logs'] . "/${file}-ALERT.log", logger::ALERT, false));             // Критические ошибки
        self::$logger->pushHandler(new StreamHandler(core::init()->path['logs'] . "/${file}-EMERGENCY.log", logger::EMERGENCY, false));     // Критические ошибки
        self::$logger->pushHandler(new StreamHandler(core::init()->path['logs'] . "/$file.log", logger::DEBUG));                            // Общий лог

        // test
        // self::$logger->pushProcessor(function ($record) {
        //     $record['extra']['dummy'] = 'Hello world!';

        //     return $record;
        // });

        echo '[' . date_format(new DateTime(core::init()->timezone), 'd-m-Y H:i:s') . '] Начало работы', PHP_EOL;
        self::$logger->info('Начало работы');

        return self::$instance;
    }

    public static function get(): ?jasmo
    {
        return self::$instance;
    }

    public static function delete(): ?jasmo
    {
        return self::$instance;
    }

    public static function postErrorHandler(): ?jasmo
    {
        /** 
         * Подключение логгера в обработчик ошибок (Jasny)
         */
        $handler = new ErrorHandler(self::$logger);

        $handler->logUncaught(E_ALL); // Обрабатывать все ошибки
        $handler->onFatalError(function ($error) {
            self::$logger->error($error);
        }, true);

        return self::$instance;
    }

    public static function deleteErrorHandler(): ?jasmo
    {
        return self::$instance;
    }

    public static function postShutdownHandler(): ?jasmo
    {
        register_shutdown_function(array(self::$instance, 'handlerShutdown'));

        return self::$instance;
    }


    public function handlerShutdown(): void
    {
        self::$logger->info('Завершение работы');
        echo '[' . date_format(new DateTime(core::init()->timezone), 'd-m-Y H:i:s') . '] Завершение работы', PHP_EOL;
    }

    // public function handlerErrors($errno, $errstr, $errfile, $errline, $errcontext): bool
    // {
    //     echo "друг, да ты вероянто напортачил $errno $errstr", PHP_EOL, PHP_EOL;
    //     if ($this->logger->error("друг, да ты вероянто напортачил $errno $errstr")) {
    //         return true;
    //     } else {
    //         return false;
    //     }
    // }
}
